<?php
require_once(__DIR__ . '/../inc/config.php');

// debug switch
$debug = true;

/*--------------------------------------------------
Setup
--------------------------------------------------*/

// if stripe live mode
if($stripe_mode == 1) {
	$stripe_key = $stripe_live_secret_key;
}

// else is stripe test mode
else {
	$stripe_key = $stripe_test_secret_key;
}

// init stripe object
$stripe = new \Stripe\StripeClient($stripe_key);

// set stripe_data_currency to lowercase
$stripe_data_currency = strtolower($stripe_data_currency);

/*--------------------------------------------------
Retrieve event
--------------------------------------------------*/

// input or payload
$input = @file_get_contents('php://input');

// event init
$event = array();

// get event id
$event_id = json_decode($input, true)['id'];

// use the event id to get the full event directly from Stripe instead of trusting the webhook data
if(!empty($event_id)) {
	$event = $stripe->events->retrieve($event_id, []);

	// convert to array
	$event = $event->toArray(true);
}

// debug
if($debug == 'stripe') {
	try {
		// mailer params
		$PHPMailer->ClearAllRecipients();
		$PHPMailer->setFrom($admin_email, $site_name);
		$PHPMailer->addAddress($dev_email);
		$PHPMailer->addReplyTo($admin_email);
		$PHPMailer->isHTML(false);
		$PHPMailer->Subject = 'Stripe Debug';
		$PHPMailer->AllowEmpty = true;
		$PHPMailer->Body = print_r($event, true);

		// send
		$PHPMailer->send();
	} catch (Exception $e) {
		echo "Message could not be sent.";
	}
}

if(empty($event)) {
	throw new Exception('Event cannot be empty.');
}

/*--------------------------------------------------
Webhook vars
--------------------------------------------------*/

// vars
$ipn_description = !empty($event['data']['object']['object']) ? $event['data']['object']['object'] : '';
$event_type      = !empty($event['type']                    ) ? $event['type']                     : '';
$payment_status  = !empty($event['data']['object']['status']) ? $event['data']['object']['status'] : '';
$event_id        = !empty($event['id']                      ) ? $event['id']                       : '';
$event_date      = !empty($event['created']                 ) ? $event['created']                  : '';

/*--------------------------------------------------
Event types
--------------------------------------------------*/

$events = array(
	'charge.succeeded',
	'charge.refunded',
	'charge.failed',
	'customer.subscription.created',
	'customer.subscription.deleted',
	'invoice.payment_succeeded',
	'invoice.payment_failed'
);

// handle only event types listed above
if(!in_array($event_type, $events)) {
	die();
}

/*--------------------------------------------------
Subscription check
--------------------------------------------------*/

$is_subscription = null;

if($event_type == 'charge.succeeded') {
	if(empty($event['data']['object']['invoice'])) {
		$is_subscription = false;
	}

	else {
		$is_subscription = true;
	}
}

if( $event_type == 'customer.subscription.created' ||
	$event_type == 'customer.subscription.deleted' ||
	$event_type == 'invoice.payment_succeeded'
	) {
	$is_subscription = true;
}

if($event_type == 'charge.failed') {
	// if charge failed, it could be either subscription or buy now
	// but if metadata is empty, it means it's subscription, if contains metadata, it's buy now
	if(!empty($event['data']['object']['metadata'])) {
		$is_subscription = false;
	}

	else {
		$is_subscription = true;
	}
}

/*--------------------------------------------------
Metadata
--------------------------------------------------*/

/*
buy now:
metadata is sent with \Stripe\Charge::create and received in webhook event type 'charge.succeeded'

subscriptions:
metadata is sent with \Stripe\Subscription::create and received in webhook event type 'customer.subscription.created' and 'customer.subscription.deleted', also in 'invoice.payment_succeeded', 'invoice.created'
*/

$plan_type = null;
$plan_id   = null;
$user_id   = null;
$uniq_id   = null;
$subscr_id = null;

if(in_array($event_type, array('charge.succeeded', 'charge.failed', 'charge.refunded'))) {
	if(!$is_subscription) {
		$plan_id = !empty($event['data']['object']['metadata']['plan_id']) ? $event['data']['object']['metadata']['plan_id']   : null;
		$user_id = !empty($event['data']['object']['metadata']['user_id']) ? $event['data']['object']['metadata']['user_id'] : null;
		$uniq_id = !empty($event['data']['object']['metadata']['uniq_id']) ? $event['data']['object']['metadata']['uniq_id'] : null;
	}
}

if($event_type == 'customer.subscription.created' || $event_type == 'customer.subscription.deleted') {
	$plan_id = !empty($event['data']['object']['metadata']['plan_id']) ? $event['data']['object']['metadata']['plan_id'] : null;
	$user_id = !empty($event['data']['object']['metadata']['user_id']) ? $event['data']['object']['metadata']['user_id'] : null;
	$uniq_id = !empty($event['data']['object']['metadata']['uniq_id']) ? $event['data']['object']['metadata']['uniq_id'] : null;
	$subscr_id = !empty($event['data']['object']['id']) ? $event['data']['object']['id'] : null;
}

if($event_type == 'invoice.payment_succeeded') {
	$plan_id = !empty($event['data']['object']['subscription_details']['metadata']['plan_id']) ? $event['data']['object']['subscription_details']['metadata']['plan_id'] : null;
	$user_id = !empty($event['data']['object']['subscription_details']['metadata']['user_id']) ? $event['data']['object']['subscription_details']['metadata']['user_id'] : null;
	$uniq_id = !empty($event['data']['object']['subscription_details']['metadata']['uniq_id']) ? $event['data']['object']['subscription_details']['metadata']['uniq_id'] : null;

	$subscr_id = !empty($event['data']['object']['id']) ? $event['data']['object']['id'] : null;
}

/*--------------------------------------------------
Customer info
--------------------------------------------------*/

// customer from webhook
$customer = $event['data']['object']['customer'];

if(!empty($customer)) {
	$customer_obj   = $stripe->customers->retrieve($customer, []);
	$customer       = $customer_obj->toArray(true);
	$customer_email = $customer['email'];
	$customer_id    = $customer['id'];
}

else {
	$customer_email = '';
	$customer_id    = '';
}

/*--------------------------------------------------
Plan details
--------------------------------------------------*/

$query = "SELECT * FROM plans WHERE plan_id = :plan_id";
$stmt = $conn->prepare($query);
$stmt->bindValue(':plan_id', $plan_id);
$stmt->execute();
$row = $stmt->fetch(PDO::FETCH_ASSOC);

$plan_price   = !empty($row['plan_price'  ]) ? $row['plan_price'  ] : null;
$plan_credits = !empty($row['plan_credits']) ? $row['plan_credits'] : '';
$plan_type    = !empty($row['plan_type'   ]) ? $row['plan_type'   ] : null;

// convert plan price to smallest unit
if(!empty($plan_price)) {
	$plan_price = filter_var($plan_price, FILTER_SANITIZE_NUMBER_FLOAT);
}

// stripe always uses non-decimal amount in minimum currency unit (e.g. cents for USD and yen for JPY)
// plan_price is always decimal so if $stripe_min_unit_is_cent is false(for eg. JPY), then divide amount by 100
if(!$stripe_min_unit_is_cent) {
	$plan_price = $plan_price / 100;
}

/*--------------------------------------------------
User details
--------------------------------------------------*/

$query = "SELECT email, first_name FROM users WHERE id = :user_id";
$stmt = $conn->prepare($query);
$stmt->bindValue(':user_id', $user_id);
$stmt->execute();
$row = $stmt->fetch(PDO::FETCH_ASSOC);

$user_email = !empty($row['email']) ? $row['email'] : $customer_email;
$user_firstname = !empty($row['first_name']) ? $row['first_name'] : '';

/*--------------------------------------------------
Amount
--------------------------------------------------*/

// init
$amount = null;
$amount_currency = null;

// get values from object data
if($plan_type == 'one_time' || $plan_type == 'one_time_feat') {
	$amount = !empty($event['data']['object']['amount']) ? $event['data']['object']['amount'] : null;
	$amount_currency = !empty($event['data']['object']['currency']) ? $event['data']['object']['currency'] : null;
}

else {
	$amount = !empty($event['data']['object']['plan']['amount']) ? $event['data']['object']['plan']['amount'] : null;
	$amount_currency = !empty($event['data']['object']['plan']['currency']) ? $event['data']['object']['plan']['currency'] : null;
}

if(empty($amount) && empty($amount_currency)) {
	$amount = !empty($event['data']['object']['lines']['data'][0]['amount']) ? $event['data']['object']['lines']['data'][0]['amount'] : null;
	$amount_currency = !empty($event['data']['object']['lines']['data'][0]['currency']) ? $event['data']['object']['lines']['data'][0]['currency'] : null;
}

if(empty($amount) && empty($amount_currency)) {
	$amount = !empty($event['data']['object']['amount']) ? $event['data']['object']['amount'] : null;
	$amount_currency = !empty($event['data']['object']['currency']) ? $event['data']['object']['currency'] : null;
}

$amount_currency = mb_strtolower($amount_currency);

/*--------------------------------------------------
Handle event
--------------------------------------------------*/

// if mode is 'payment'
if(!$is_subscription) {
	if($event_type == 'charge.succeeded') {
		// verify amount
		if($plan_price == $amount && $stripe_data_currency == $amount_currency) {
			// check if this webhook was not sent before
			$query = "SELECT COUNT(*) AS c FROM credits WHERE user = :userid AND uniq_id = :uniq_id";
			$stmt = $conn->prepare($query);
			$stmt->bindValue(':uniq_id', $uniq_id);
			$stmt->bindValue(':userid', $user_id);
			$stmt->execute();
			$row = $stmt->fetch(PDO::FETCH_ASSOC);
			$total_count = !empty($row['c']) ? $row['c'] : 0;

			// add credits
			if($total_count < 1) {
				$query = 'INSERT INTO credits(user, plan, credits, uniq_id, paym_email, gateway) VALUES(:user, :plan, :credits, :uniq_id, :paym_email, :gateway)';
				$stmt = $conn->prepare($query);
				$stmt->bindValue(':user', $user_id);
				$stmt->bindValue(':plan', $plan_id);
				$stmt->bindValue(':credits', $plan_credits);
				$stmt->bindValue(':uniq_id', $uniq_id);
				$stmt->bindValue(':paym_email', $user_email);
				$stmt->bindValue(':gateway', 'stripe');
				$stmt->execute();

				// get email template
				$query = "SELECT * FROM email_templates WHERE type = 'credits_added'";
				$stmt = $conn->prepare($query);
				$stmt->execute();
				$row = $stmt->fetch(PDO::FETCH_ASSOC);
				$email_subject = !empty($row['subject']) ? $row['subject'] : '';
				$email_body    = !empty($row['body'   ]) ? $row['body'   ] : '';

				// string replacements
				$email_subject = str_replace('%site_name%', $site_name, $email_subject);
				$email_subject = str_replace('%site_url%', $baseurl, $email_subject);
				$email_body = str_replace('%site_name%', $site_name, $email_body);
				$email_body = str_replace('%site_url%', $baseurl, $email_body);

				// send email
				try {
					// mailer params
					$PHPMailer->clearAllRecipients();
					$PHPMailer->clearReplyTos();
					$PHPMailer->AllowEmpty = true;
					$PHPMailer->setFrom($admin_email, $site_name);
					$PHPMailer->addAddress($user_email);
					$PHPMailer->addReplyTo($admin_email);
					$PHPMailer->isHTML(false);
					$PHPMailer->Subject = $email_subject;
					$PHPMailer->Body = $email_body;

					// send
					$PHPMailer->send();
				} catch (Exception $e) {
					echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
				}
			}

			else {
				echo 'webhook already received';
			}
		}

		else {
			$debug_msg .= "\n---$plan_price <> $amount --- $stripe_data_currency <> $amount_currency";
			// send email to admin about plan price not matching amount paid
			try {
				// mailer params
				$PHPMailer->clearAllRecipients();
				$PHPMailer->clearReplyTos();
				$PHPMailer->AllowEmpty = true;
				$PHPMailer->setFrom($admin_email, $site_name);
				$PHPMailer->addAddress($dev_email);
				$PHPMailer->addReplyTo($admin_email);
				$PHPMailer->isHTML(false);
				$PHPMailer->Subject = 'charge.succeeded - plan price <> amount';
				$PHPMailer->Body = $debug_msg;

				// send
				$PHPMailer->send();
			} catch (Exception $e) {
				echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
			}
		}
	}

	else if($event_type == 'charge.failed') {
		// define email informing user of charge.failed
		$query = "SELECT * FROM email_templates WHERE type = 'charge_failed'";
		$stmt = $conn->prepare($query);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		$email_subject = !empty($row['subject']) ? $row['subject'] : '';
		$email_body    = !empty($row['body'   ]) ? $row['body'   ] : '';

		// string replacements
		$email_subject = str_replace('%site_name%', $site_name, $email_subject);
		$email_subject = str_replace('%site_url%', $baseurl, $email_subject);
		$email_body = str_replace('%site_name%', $site_name, $email_body);
		$email_body = str_replace('%site_url%', $baseurl, $email_body);

		// send email
		try {
			// mailer params
			$PHPMailer->clearAllRecipients();
			$PHPMailer->clearReplyTos();
			$PHPMailer->AllowEmpty = true;
			$PHPMailer->setFrom($admin_email, $site_name);
			$PHPMailer->addAddress($user_email);
			$PHPMailer->addReplyTo($admin_email);
			$PHPMailer->isHTML(false);
			$PHPMailer->Subject = $email_subject;
			$PHPMailer->Body = $email_body;

			// send
			$PHPMailer->send();
		} catch (Exception $e) {
			echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
		}
	}

	else if($event_type == 'charge.refunded') {
		// remove credits
		$query = 'DELETE FROM credits WHERE user = :user AND plan = :plan AND uniq_id = :uniq_id';
		$stmt = $conn->prepare($query);
		$stmt->bindValue(':user', $user_id);
		$stmt->bindValue(':plan', $plan_id);
		$stmt->bindValue(':uniq_id', $uniq_id);
		$stmt->execute();

		// update listings status to unpaid
		$query = 'UPDATE listings SET paid = 0 WHERE userid = :userid AND plan = :plan AND credits_uniq_id = :uniq_id';
		$stmt = $conn->prepare($query);
		$stmt->bindValue(':userid', $user_id);
		$stmt->bindValue(':plan', $plan_id);
		$stmt->bindValue(':uniq_id', $uniq_id);
		$stmt->execute();
	}

	else {
		// log transaction
		error_log("Webhook data is a one-off purchase but charge neither succeeded nor failed/refunded");
	}
}

// else if mode is 'subscription'
else {
	// webhook types to act on:
		// 'customer.subscription.created'
		// 'customer.subscription.deleted'
		// 'invoice.payment_succeeded'
		// 'invoice.payment_failed'
	// if subscription succeeded
	if($event_type == 'customer.subscription.created') {
		// verify amount
		if($plan_price == $amount && $stripe_data_currency == $amount_currency) {
			// check if this webhook was not sent before
			$query = "SELECT COUNT(*) AS c FROM credits WHERE user = :userid AND uniq_id = :uniq_id";
			$stmt = $conn->prepare($query);
			$stmt->bindValue(':uniq_id', $uniq_id);
			$stmt->bindValue(':userid', $user_id);
			$stmt->execute();
			$row = $stmt->fetch(PDO::FETCH_ASSOC);
			$total_count = !empty($row['c']) ? $row['c'] : 0;

			// add credits
			if($total_count < 1) {
				$query = 'INSERT INTO credits(user, plan, credits, uniq_id, paym_email, gateway, subscr_id) VALUES(:user, :plan, :credits, :uniq_id, :paym_email, :gateway, :subscr_id)';
				$stmt = $conn->prepare($query);
				$stmt->bindValue(':user', $user_id);
				$stmt->bindValue(':plan', $plan_id);
				$stmt->bindValue(':credits', $plan_credits);
				$stmt->bindValue(':uniq_id', $uniq_id);
				$stmt->bindValue(':paym_email', $customer_email);
				$stmt->bindValue(':gateway', 'stripe');
				$stmt->bindValue(':subscr_id', $subscr_id);
				$stmt->execute();

				// subscr_signup email
				$query = "SELECT * FROM email_templates WHERE type = 'subscr_signup'";
				$stmt = $conn->prepare($query);
				$stmt->execute();
				$row = $stmt->fetch(PDO::FETCH_ASSOC);
				$email_subject = !empty($row['subject']) ? $row['subject'] : '';
				$email_body    = !empty($row['body'   ]) ? $row['body'   ] : '';

				// string replacements
				$email_subject = str_replace('%site_name%', $site_name, $email_subject);
				$email_subject = str_replace('%site_url%', $baseurl, $email_subject);
				$email_body = str_replace('%site_name%', $site_name, $email_body);
				$email_body = str_replace('%site_url%', $baseurl, $email_body);

				// send email
				try {
					// mailer params
					$PHPMailer->clearAllRecipients();
					$PHPMailer->clearReplyTos();
					$PHPMailer->AllowEmpty = true;
					$PHPMailer->setFrom($admin_email, $site_name);
					$PHPMailer->addAddress($user_email);
					$PHPMailer->addReplyTo($admin_email);
					$PHPMailer->isHTML(false);
					$PHPMailer->Subject = $email_subject;
					$PHPMailer->Body = $email_body;

					// send
					$PHPMailer->send();
				} catch (Exception $e) {
					echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
				}
			}

			else {
				echo 'webhook already received';
			}
		}

		else {
			// send email to admin about diverging price
			$debug_msg .= "\n---$plan_price <> $amount --- $stripe_data_currency <> $amount_currency";

			try {
				// mailer params
				$PHPMailer->clearAllRecipients();
				$PHPMailer->clearReplyTos();
				$PHPMailer->AllowEmpty = true;
				$PHPMailer->setFrom($admin_email, $site_name);
				$PHPMailer->addAddress($dev_email);
				$PHPMailer->addReplyTo($admin_email);
				$PHPMailer->isHTML(false);
				$PHPMailer->Subject = 'charge.succeeded - plan price <> amount';
				$PHPMailer->Body = $debug_msg;

				// send
				$PHPMailer->send();
			} catch (Exception $e) {
				echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
			}
		}
	}

	else if($event_type == 'charge.succeeded') {
		// log transaction
	}

	else if($event_type == 'invoice.payment_succeeded') {
		// log transaction
	}

	else if($event_type == 'invoice.payment_failed') {
		// define email informing user of invoice.payment_failed
		$query = "SELECT * FROM email_templates WHERE type = 'subscr_failed'";
		$stmt = $conn->prepare($query);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		$email_subject = !empty($row['subject']) ? $row['subject'] : '';
		$email_body    = !empty($row['body'   ]) ? $row['body'   ] : '';

		// string replacements
		$email_subject = str_replace('%site_name%', $site_name, $email_subject);
		$email_subject = str_replace('%site_url%', $baseurl, $email_subject);
		$email_body = str_replace('%site_name%', $site_name, $email_body);
		$email_body = str_replace('%site_url%', $baseurl, $email_body);

		// send email
		try {
			// mailer params
			$PHPMailer->clearAllRecipients();
			$PHPMailer->clearReplyTos();
			$PHPMailer->AllowEmpty = true;
			$PHPMailer->setFrom($admin_email, $site_name);
			$PHPMailer->addAddress($user_email);
			$PHPMailer->addReplyTo($admin_email);
			$PHPMailer->isHTML(false);
			$PHPMailer->Subject = $email_subject;
			$PHPMailer->Body = $email_body;

			// send
			$PHPMailer->send();
		} catch (Exception $e) {
			echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
		}
	}

	else if($event_type == 'customer.subscription.deleted') {
		// remove credits
		$query = 'DELETE FROM credits WHERE user = :user AND plan = :plan AND uniq_id = :uniq_id';
		$stmt = $conn->prepare($query);
		$stmt->bindValue(':user', $user_id);
		$stmt->bindValue(':plan', $plan_id);
		$stmt->bindValue(':uniq_id', $uniq_id);
		$stmt->execute();

		// update listings status to unpaid
		$query = 'UPDATE listings SET paid = 0 WHERE userid = :userid AND plan = :plan AND credits_uniq_id = :uniq_id';
		$stmt = $conn->prepare($query);
		$stmt->bindValue(':userid', $user_id);
		$stmt->bindValue(':plan', $plan_id);
		$stmt->bindValue(':uniq_id', $uniq_id);
		$stmt->execute();

		// email user informing subscription ended
		$query = "SELECT * FROM email_templates WHERE type = 'subscr_eot'";
		$stmt = $conn->prepare($query);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		$email_subject = !empty($row['subject']) ? $row['subject'] : '';
		$email_body    = !empty($row['body'   ]) ? $row['body'   ] : '';

		// string replacements
		$email_subject = str_replace('%site_name%', $site_name, $email_subject);
		$email_subject = str_replace('%site_url%', $baseurl, $email_subject);
		$email_body = str_replace('%site_name%', $site_name, $email_body);
		$email_body = str_replace('%site_url%', $baseurl, $email_body);

		// send email
		try {
			// mailer params
			$PHPMailer->clearAllRecipients();
			$PHPMailer->clearReplyTos();
			$PHPMailer->AllowEmpty = true;
			$PHPMailer->setFrom($admin_email, $site_name);
			$PHPMailer->addAddress($user_email);
			$PHPMailer->addReplyTo($admin_email);
			$PHPMailer->isHTML(false);
			$PHPMailer->Subject = $email_subject;
			$PHPMailer->Body = $email_body;

			// send
			$PHPMailer->send();
		} catch (Exception $e) {
			echo "Message could not be sent. Mailer Error: {$PHPMailer->ErrorInfo}";
		}
	}

	else {
		// log transaction
	}
}

/*--------------------------------------------------
INSERT INTO TRANSACTIONS TABLE
--------------------------------------------------*/

$query = "INSERT INTO transactions(
		txn_type,
		uniq_id,
		user,
		paym_email,
		gateway,
		amount,
		txn_data
	)
	VALUES(
		:txn_type,
		:uniq_id,
		:user_id,
		:paym_email,
		:gateway,
		:amount,
		:txn_data
	)";

$stmt = $conn->prepare($query);

$stmt->bindValue(':txn_type'   , $event_type);
$stmt->bindValue(':uniq_id'    , $uniq_id);
$stmt->bindValue(':user_id'    , $user_id);
$stmt->bindValue(':paym_email' , $customer_email);
$stmt->bindValue(':gateway'    , 'stripe');
$stmt->bindValue(':amount'     , $amount);
$stmt->bindValue(':txn_data'   , $event_id);

$stmt->execute();
